// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// This is the file to include in your code if you want access to the
// complete AStack template class

// First, get the declaration for the base stack class
#include "stack.h"

// This is the declaration for AStack.
// Array-based stack implementation
template <typename E>
class AStack: public Stack<E>
{
    private:
        int maxSize;     // Maximum size of stack
        int topIndex;    // Index for top element
        E *listArray;    // Pointer to type E to hold array of stack elements

    public:
        // Constructor
        AStack(int size = defaultSize)
        {
            maxSize = size;
            topIndex = 0;
            listArray = new E[size];
        }

        // Destructor
        ~AStack()
        {
            delete[] listArray;
        }

        // Reinitialize
        void clear()
        {
            topIndex = 0;
        }

        // Put "it" on stack
        void push(const E &it)
        {
            Assert(topIndex != maxSize, "Stack is full");
            listArray[topIndex++] = it;
        }

        // Pop top element
        E pop()
        {
            Assert(topIndex != 0, "Stack is empty");
            return listArray[--topIndex];
        }

        // Return top element
        const E & peek() const
        {
            Assert(topIndex != 0, "Stack is empty");
            return listArray[topIndex-1];
        }

        // Return length
        int length() const
        {
            return topIndex;
        }
};
